/**
Copyright (C) 2004, 2005, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <string.h>

#ifndef FALSE
#define FALSE (0)
#endif
#ifndef TRUE
#define TRUE (1)
#endif

class QString {               // new string definition, (see QT on www.trolltech.com)

public:
       char *string;

       QString(char *s="") {                  // standard constructor
           string=new char[strlen(s)+1];
           strcpy(string,s);
       }

       QString operator=(char *s) {           // copy constructor
           string=new char[strlen(s)+1];
           strcpy(string,s);
           return(*this);
       }

       QString operator=(QString s) {           // copy constructor
           string=new char[strlen(s.string)+1];
           strcpy(string,s.string);
           return(*this);
       }

       QString operator+=(char *s) {           // concatenation constructor
           char *str=new char[strlen(s)+strlen(string)+1];
           strcpy(str,string);
           strcat(str,s);
           string=str;
           return(*this);
       }

       QString operator+=(QString s) {           // concatenation constructor
           char *str=new char[strlen(s.string)+strlen(string)+1];
           strcpy(str,string);
           strcat(str,s.string);
           string=str;
           return(*this);
       }

       bool operator==(char *s) {              // compare constructor
           if (strcmp(string,s)) return (FALSE);
           else return (TRUE);
       }

       bool operator==(QString s) {              // compare constructor
           if (strcmp(string,s.data())) return (FALSE);
           else return (TRUE);

       }

       bool operator!=(char *s) {              // compare constructor
           if (strcmp(string,s)) return (TRUE);
           else return (FALSE);

       }

       unsigned length(void) {               // length of the string
            return (strlen(string));
       }

       char *data(void) {               // returns string as char*
            return (string);
       }

       int find(char c, int n=0) {      // finds char c at position n
           int pos=0;
           char *s=string;
           if (n<0 || (unsigned)n>strlen(string)) return (-1);
           while (n && *s) { s++; n--; }
           do {
              if (*s==c) return(pos);
              s++;
              pos++;
           } while(*s);
           return(-1);               // -1 means not found
       }

       int find(QString sf, int n=0) {      // finds string sf at position n
           int pos=n;
           char *s=string;
           QString temp;

           if (n<0 || (unsigned)n>strlen(string) || sf.length()>strlen(string)) return (-1);
           while (n && *s) { s++; n--; }
           do {
              QString temp=s;
              temp=temp.left(sf.length());
              if (temp==sf) return(pos);
              s++;
              pos++;
           } while(*s && strlen(s) >= sf.length());
           return(-1);               // -1 means not found
       }

       QString left(int n) {      // returns the n-th left chars
           QString str=string;
           char *s=str.string;
           if (n<0 || (unsigned)n>=strlen(string)) return (str);
           while (n && *s) { s++; n--; }
           *s='\0';
           return(str);
       }

       QString mid(int n, int w) {   // returns string with w chars at position n
           QString str=string;
           char *s=str.string;
           if (n<0 || w<0) return (str);
           while (n && *s) { s++; n--; }
           str.string=s;
           while (w && *s) { s++; w--; }
           *s='\0';
           return(str);
       }

       QString right(int n) {    // returns the n-th right chars
           QString str=string;
           char *s=str.string+str.length();
           if (n<0 || (unsigned)n>=strlen(string)) return (str);
           while (n) { s--; n--; }
           str.string=s;
           return(str);
       }

       int contains(char c) {     // returns numbers of c contained in the string
           int pos=0;
           char *s=string;
           do {
              if (*s==c) pos++;
              s++;
           } while(*s);
           return(pos);
       }

       void free(void) {       // deletes all special signs like line break, etc.
           char *s=string;
           do {
              if (*s=='\r' || *s=='\n') { *s='\0'; break; }  // if found then end string.
              s++;
           } while(*s);
       }

       int free2(void) {                 // changes the string for further work
           bool space=TRUE;              // marks a space
           int i=0, col=0;               // col is needed for columns remembering if some spaces were deleted before
           char *temp=new char[256];     // new string
           char *t=temp;
           char *s=string;
           do {
              switch (*s) {
                case '\r':
                case '\n': *s='\0';
                           if (!col) i=1;
                           break;       // deletes all special signs like line break, etc.
                case ' ' :
                case '\t': if (!space) {
                             *t=' ';   // tabs will be changed into spaces
                             t++;
                             col++;
                             space=TRUE;   // multiple spaces will be deleted
                           }
                           break;
                default  : *t=*s;
                           t++;
                           col++;
                           space=FALSE;
                           break;
              }
              s++;         //
           } while(*s);
           if (*(t-1)==' ') t--;       // space at last position will be deleted
           *t='\0';                    // string finished
           // printf("|%s|\n", temp);
           string=temp;
           return (i);
       }
};
